CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000 

  ANT_PIN       = 3
  FB_PIN        = 4
  IF_PIN        = 8
  QF_PIN        = 9
  LED_PIN       = 26
  FSAMPLE       = 1000
  CLKS          = 80_000_000 / FSAMPLE
  MCLKS         = CLKS / 2
  SIG_THLD      = 5000

OBJ

  u : "umath"
  pst : "parallax serial terminal"

VAR

  long  signal, frequency
               
PUB Start | tune_freq

  pst.start(115200)

  dira[FB_PIN]~~
  ctra := %01001 << 26 | FB_PIN << 9 | ANT_PIN
  frqa := 1
  q_freq := @frequency
  d_signal := @signal
  cognew(@quadrature, 0)
  cognew(@demod, 0)
  tune_freq := 600_000                                  'Change to a clear frequency in the AM band.
  frequency := u.div(tune_freq, 0, clkfreq)
  dira[LED_PIN]~~
  repeat
    pst.dec(signal)
    pst.char(13)
    outa[LED_PIN] := (signal > SIG_THLD)
    waitcnt(cnt + clkfreq / 10)
    
DAT

'' Generate two square waves, 90 degrees out of phase.

              org       0
quadrature    mov       dira,q_dira0
              mov       ctra,q_ctra0
              mov       ctrb,q_ctrb0
              
:query        rdlong    freq,q_freq
              cmp       freq,frqa wz
        if_z  jmp       #:query

              mov       frqa,#0
              mov       frqb,#0
              mov       phsa,#0
              mov       phsb,_4000_0000
              add       phsb,freq
              add       phsb,freq
              add       phsb,freq
              add       phsb,freq
              mov       frqa,freq
              mov       frqb,freq
              jmp       #:query

q_freq        long      0-0
q_ctra0       long      %00100 << 26 | IF_PIN
q_ctrb0       long      %00100 << 26 | QF_PIN
q_dira0       long      1 << IF_PIN | 1 << QF_PIN

_4000_0000    long      $4000_0000

freq          res       1

''===================================================================

'' I and Q mixers and detector.

              org       0
demod         mov       frqa,#1
              mov       frqb,#1
              mov       ctra,d_ctra0
              mov       ctrb,d_ctrb0
              mov       time,cnt
              add       time,sample

:mix          waitcnt   time,sample
              mov       i_samp,phsa
              mov       phsa,#0
              mov       q_samp,phsb
              mov       phsb,#0
              mov       acc,i_samp
              call      #square
              mov       i_samp,acc
              mov       acc,q_samp
              call      #square
              add       acc,i_samp
              call      #sqrt
              wrlong    acc,d_signal
              jmp       #:mix
              
''--------[square]---------------------------------------------------

'' acc = (acc - half) ** 2   (Multiply routine from "Propeller Guts".)

square        sub       acc,half
              abs       acc,acc
              mov       ctr,#16
              mov       accx,acc
              shl       accx,#16
              shr       acc,#1 wc
              
:sqlp   if_c  add       acc,accx wc
              rcr       acc,#1 wc
              djnz      ctr,#:sqlp

square_ret    ret

''--------[sqrt]-----------------------------------------------------

'' acc = sqrt(acc)      (Square root routine from "Propeller Guts".)

sqrt          mov       accy,#0
              mov       accx,#0
              mov       ctr,#16
:rtlp         shl       acc,#1 wc

              rcl       accy,#1
              shl       acc,#1 wc
              rcl       accy,#1
              shl       accx,#2 
              or        accx,#1
              cmpsub    accy,accx wc
              shr       accx,#2
              rcl       accx,#1
              djnz      ctr,#:rtlp

              mov       acc,accx
sqrt_ret      ret

''-------------------------------------------------------------------

d_signal      long      0-0
d_ctra0       long      %10110 << 26 | IF_PIN << 9 | FB_PIN
d_ctrb0       long      %10110 << 26 | QF_PIN << 9 | FB_PIN
sample        long      CLKS
half          long      MCLKS

acc           res       1
accx          res       1
accy          res       1
ctr           res       1
i_samp        res       1
q_samp        res       1
time          res       1

  